/*
  Firmware for Bitmap Projector
  Copyright (C) 2011 Matt Evans

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  General hardware definitions, GPIOs etc.
*/

#ifndef HW_H
#define HW_H

// This selects the right IO defs from the -mmcu=atmega8 GCC param.
#include <avr/io.h>

// Define CPU frequency before including delay.h
#define F_CPU (16000000UL)
// This is (F_CPU / (16*baud))-1.  See p.195 of atmega48 doc2545.
#define BAUDREG ((F_CPU / (16 * 9600UL)) - 1)

#define B(x) (1<<(x))

#define IO_D_LED  B(PD4)
#define LED_ON() do { PORTD |= IO_D_LED; } while(0)
#define LED_OFF() do { PORTD &= ~IO_D_LED; } while(0)
#define IO_FAN    B(PD7)
#define IO_LAMP   B(PB1)
#define FAN_ON() do { PORTD |= IO_FAN; } while(0)
#define FAN_OFF() do { PORTD &= ~IO_FAN; } while(0)
#define LAMP_ON() do { PORTB &= ~IO_LAMP; } while(0)
#define LAMP_OFF() do { PORTB |= IO_LAMP; } while(0)

#endif
