/*
  Firmware for Bitmap Projector
  Copyright (C) 2011 Matt Evans

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  Very simple serial IO routines
*/

#include "hw.h"
#include <avr/io.h>
#include <avr/pgmspace.h>
#include "serio.h"

void serial_init(void)
{
  // Set up serial stuff, baudrate and 8N1
  UCSR0A = 0;
  UCSR0B = (1<<TXEN0) | (1<<RXEN0);
  UCSR0C = (1<<UCSZ01) | (1<<UCSZ00);
  UBRR0 = BAUDREG;
}

void putchr(char c)
{
  loop_until_bit_is_set(UCSR0A, UDRE0);
  UDR0 = c;
}

uint8_t rxWaiting(void)
{
  return (UCSR0A & (1<<RXC0));
}

char getchr(void)
{
  loop_until_bit_is_set(UCSR0A, RXC0);
  return UDR0;
}

void pnewl(void)
{
  putchr('\r');
  putchr('\n');
}

void printstr(const char *str)
{
  char c;
  while ((c = *str++) != 0)
  {
    if (c == '\n')
    {
      putchr('\r');
    }
    putchr(c);
  }
}

void printstr_p(const char *str)
{
  char c;

  while ((c = pgm_read_byte(str++)) != 0)
  {
    if (c == '\n')
    {
      putchr('\r');
    }
    putchr(c);
  }
}


void phex8(uint8_t n)
{
  uint8_t hi = (n >> 4);
  putchr(hi + ( (hi > 9) ? ('a' - 10) : '0' ));
  n = n & 0xf;
  putchr(n + ( (n > 9) ? ('a' - 10) : '0' ));
}

void phex16(uint16_t n)
{
  phex8(n >> 8);
  phex8(n & 0xff);
}

void phex32(uint32_t n)
{
  phex16(n >> 16);
  phex16(n & 0xffff);
}

/* This is ugly! */
uint8_t  ghex8(unsigned char *errPtr)
{
  uint8_t ret = 0;
  char hn, ln;

  hn = getchr();
  char hnU = hn | 0x20;
  if (hn >= '0' && hn <= '9')
  {
    ret = ( (hn - '0') << 4 );
  }
  else if (hnU >= 'a' && hnU <= 'f')
  {
    ret = ( (hn - 'a' + 10) << 4 );
  }
  else 
  {
    *errPtr = 1;
  }
  
  ln = getchr();
  char lnU = ln | 0x20;
  if (ln >= '0' && ln <= '9')
  {
    ret |= (ln - '0');
  }
  else if (lnU >= 'a' && lnU <= 'f')
  {
    ret |= (ln - 'a' + 10);
  }
  else 
  {
    *errPtr = 1;
  }

  return ret;
}

uint16_t ghex16(unsigned char *errPtr)
{
  uint8_t hi, lo = 0;

  hi = ghex8(errPtr);
  if (!*errPtr)
  {
    lo = ghex8(errPtr);
  }

  return ((uint16_t)hi << 8) | lo;
}

uint32_t ghex32(unsigned char *errPtr)
{
  uint16_t a = 0;
  uint16_t b = 0;

  a = ghex16(errPtr);
  if (!*errPtr)
  {
    b = ghex16(errPtr);
  }
  return ((uint32_t)a << 16) | (b);
}
